@@ -1,6 +1,6 @@
 from .core import Primitive, synchronized, PyThread, gated, TimerThread, Timeout
 from .dequeue import DEQueue
-from .TaskQueue import TaskQueue, Task
+from .task_queue import TaskQueue, Task
 from .Scheduler import Scheduler
 from .Subprocess import Subprocess, ShellCommand
 from .RWLock import RWLock
@@ -8,13 +8,14 @@ from .Version import Version
 from .promise import Promise
 from .processor import Processor
 from .flag import Flag
+from .gate import Gate
 
 
 __version__ = Version
 version_info = tuple(Version.split("."))
 
 
-__all__ = [
+__a_ll__ = [
     'Primitive',
     'PyThread',
     'TimerThread',
@@ -28,5 +29,6 @@ __all__ = [
     'Version', '__version__', 'version_info',
     'Timeout',
     'Promise',
-    'Scheduler'
+    'Scheduler',
+    'Gate'
 ]
